import argparse
import json
import sys
import xml.sax.saxutils
import xml.etree.ElementTree

def read_ann(fi):
    # Read the entire data first.
    n = 0
    D = {}
    for line in fi:
        n += 1
        if line.startswith('#'):
            continue
        fields = line.strip('\n').split('\t')
        assert len(fields) == 3, '[ANN] Field length must be three at lines {}.'.format(n)
        assert fields[0] not in D, '[ANN] Duplicated IDs at lines {}'.format(n)
        D[fields[0]] = (fields[1].replace(';', ' ').split(' '), fields[2])

    # Build link information.
    L = {}
    for key, (value1, value2) in D.iteritems():
        if key.startswith('T'):
            L[key] = dict(
                begin=int(value1[1]),
                end=int(value1[-1]),
                mention=value2,
                type=value1[0],
                )
    
    # Fill references.
    for key, (value1, value2) in D.iteritems():
        if key.startswith('N'):
            ref = value1[1]
            assert value1[0] == 'Reference', '[ANN] No reference in N tag'
            assert ref in L, '[ANN] Duplicated IDs for normalization'
            assert value1[2].startswith('Wiki:'), '[ANN] Link must starts with "Wiki:"'
            L[ref]['wiki'] = 'ja:'+value1[2][5:]
            L[ref]['title'] = value2
    
    return L

def merge(xmlfile, annfile):
    L = read_ann(annfile)
    T = unicode(xmlfile.read())
    X = [''] * len(T)

    for link in L.itervalues():
        begin = link['begin']
        assert T[begin-1] == '>', 'The character before the begin offset must be ">": {}'.format(begin)
        assert link['type'] in ('A', 'Ambig', 'NIL'), 'Unsupported annotation label: {}'.format(link['type'])

        if link['type'] in ('A', 'Ambig'):
            assert 'wiki' in link, 'No Wiki link: {}'.format(json.dumps(link, ensure_ascii=False))
            assert 'title' in link, 'No Wiki title: {}'.format(json.dumps(link, ensure_ascii=False))
            attr = ' wiki="{}" title={}>'.format(
                link['wiki'],
                xml.sax.saxutils.quoteattr(link['title'])
                )
        elif link['type'] == 'NIL':
            attr = ' wiki="NIL">'
        X[begin-1] = attr

    s = ''
    for i in range(len(T)):
        if X[i]:
            s += X[i]
        else:
            s += T[i]

    #tree = xml.etree.ElementTree.fromstring(s)
    return s

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='Merge an annotation (ann) with the text (xml).')
    parser.add_argument(
        'src', metavar='INPUT', type=str,
        help='the original text in XML format'
    )
    parser.add_argument(
        'ann', metavar='ANN', type=str,
        help='the annotation file (ann)'
    )
    parser.add_argument(
        '-o', '--output', metavar='OUTPUT', type=str,
        help='the output file (default: STDOUT)'
    )
    args = parser.parse_args()
    
    fo = open(args.output, 'w') if args.output else sys.stdout
    fo.write(merge(open(args.src), open(args.ann)))
